#!/bin/ash
#compile nvidia kernel modules, xorg drivers, GL libraries, create sfs.

. /etc/DISTRO_SPECS
NVVER='525.125.06'
KERNELVER="`uname -r`"
DASHOUT="`find /usr/src -maxdepth 1 -type d -name "out-k${KERNELVER}-*"`"
if [ "$DASHOUT" ];then
 POSTVER="-`echo -n "$DASHOUT" | rev | cut -f 1 -d '-' | rev`" #ex: -43
else
 POSTVER=""
fi

echo
SFSPATH="/mnt/build/bk/www/ibiblio/easyos/${DISTRO_TARGETARCH}/packages/sfs/easyos/${DISTRO_BINARY_COMPAT}/${DISTRO_COMPAT_VERSION}"
if [ ! -d ${SFSPATH} ];then
 echo "Path ${SFSPATH} does not exist"
 exit
fi

if [ ! -f NVIDIA-Linux-x86_64-${NVVER}.run ];then
 echo '/usr/src/NVIDIA-Linux-x86_64-${NVVER}.run does not exist, aborting'
 exit
fi
chmod 755 NVIDIA-Linux-x86_64-${NVVER}.run

rm -rf /opt/* 2>/dev/null
rm -rf out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64 2>/dev/null

#rem:  --no-install-libglvnd
CC=gcc ./NVIDIA-Linux-x86_64-${NVVER}.run --force-selinux=no --run-nvidia-xconfig --expert --opengl-prefix=/opt --utility-prefix=/opt --documentation-prefix=/opt --no-nvidia-modprobe --kernel-name=${KERNELVER} --no-rpms --no-x-check --no-nouveau-check --no-dkms --no-unified-memory --no-peermem --concurrency-level=1 --skip-depmod --no-systemd --no-questions --ui=none --no-check-for-alternate-installs --no-kernel-module-source --compat32-libdir=lib32 --compat32-prefix=/opt --no-distro-scripts --glvnd-egl-config-path=/opt/share/glvnd/egl_vendor.d --x-sysconfig-path=/usr/share/X11/xorg.conf.d

#this got changed, restore:
cp -a -f --remove-destination /mnt/.easy_ro/easy_sfs/usr/lib/libGL.so.1 /usr/lib/

mkdir -p out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64
cp -a /opt out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/
mkdir -p out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/lib
cp -a /usr/lib/firmware/nvidia out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/lib/
mkdir -p out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/lib/modules/${KERNELVER}
cp -a /mnt/.easy_rw/mainrw/usr/lib/modules/${KERNELVER}/kernel out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/lib/modules/${KERNELVER}

cp -a /usr/lib/nvidia out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/lib/

cp -a /mnt/.easy_rw/mainrw/usr/lib/xorg out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/lib/

mkdir -p out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/applications
cp -a /usr/share/applications/nvidia-settings.desktop out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/applications/

cp -a /mnt/.easy_rw/mainrw/usr/share/egl out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/
cp -a /usr/share/nvidia out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/

mkdir -p out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/glvnd
mkdir -p out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/X11
ln -s /opt/share/glvnd/egl_vendor.d  out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/glvnd/egl_vendor.d
#ln -s /opt/share/X11/xorg.conf.d out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/X11/xorg.conf.d
#ln -s /opt/lib/libOpenGL.so.0 out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/opt/lib/libGL.so.1

mkdir -p out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/etc/profile.d
echo 'export PATH="/opt/bin:${PATH}"
export LD_LIBRARY_PATH="/opt/lib"' > out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/etc/profile.d/nvidia

#blacklist nouveau module (ref: /etc/rc.d/rc.sysinit creates blacklist.conf)...
mkdir -p out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/etc/rc.d
cp -a /etc/rc.d/MODULESCONFIG out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/etc/rc.d/
sed -i -e 's%^SKIPLIST=" %SKIPLIST=" nouveau %' out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/etc/rc.d/MODULESCONFIG

#hack to keep apps working...
#cp -a -f --remove-destination /mnt/.easy_ro/easy_sfs/usr/lib/libGL.so.1 /usr/lib/
ln -snf /opt/lib/libGL.so.1 out-k${KERNELVER}${POSTVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/lib/libGL.so.1
sync
chmod 755 /mnt/.easy_rw/mainrw/usr/lib/.wh.lib*
rm -f /mnt/.easy_rw/mainrw/usr/lib/.wh.lib*

#create sfs...
cd out-k${KERNELVER}${POSTVER}
echo "Creating meta-data for nvidia_${KERNELVER}-${NVVER}_amd64.sfs ..."
SIZEK="$(du -s -k nvidia_${KERNELVER}-${NVVER}_amd64 | cut -f 1)"
echo "#generated by 5nvidia.sh:
DB_pkgname='nvidia_${KERNELVER}-${NVVER}_amd64'
DB_nameonly='nvidia'
DB_version='${KERNELVER}'
DB_revision='${NVVER}'
DB_arch='amd64'
DB_categories='Utility'
DB_size='${SIZEK}'
DB_fullfilename='nvidia_${KERNELVER}-${NVVER}_amd64.sfs'
DB_dependencies='easy'
DB_description='Official NVIDIA GPU support'
DB_description_long='Kernel modules and libraries for NVIDIA GPUs'
DB_compileddistro='${DISTRO_BINARY_COMPAT}'
DB_compiledrelease='${DISTRO_COMPAT_VERSION}'
DB_path='easyos/${DISTRO_BINARY_COMPAT}/${DISTRO_COMPAT_VERSION}'
DB_apps='nvidia'
DB_icons='nvidia'
DB_install='individual'
DB_hash=''" > nvidia_${KERNELVER}-${NVVER}_amd64.specs

mkdir -p nvidia_${KERNELVER}-${NVVER}_amd64/.control
cp -a nvidia_${KERNELVER}-${NVVER}_amd64.specs nvidia_${KERNELVER}-${NVVER}_amd64/.control/nvidia.specs
mkdir -p nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/pixmaps
ffmpeg -i /opt/share/doc/NVIDIA_GLX-1.0/nvidia-settings.png -vf scale=48:48 nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/pixmaps/nvidia48.png
cp -a nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/pixmaps/nvidia48.png nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/pixmaps/nvidia.png

sync
mksquashfs nvidia_${KERNELVER}-${NVVER}_amd64 nvidia_${KERNELVER}-${NVVER}_amd64.sfs -comp zstd
sync
chmod 644 nvidia_${KERNELVER}-${NVVER}_amd64.sfs

HASH="$(sha256sum nvidia_${KERNELVER}-${NVVER}_amd64.sfs | cut -f 1 -d ' ')"
sed -i -e "s%^DB_hash=.*%DB_hash='${HASH}'%" nvidia_${KERNELVER}-${NVVER}_amd64.specs

sync
echo
echo "MUST SHUTDOWN WITHOUT SAVING SESSION!!!"

echo -n "Press ENTER to install SFS: "
read goon
echo
echo "nvidia_${KERNELVER}-${NVVER}_amd64.sfs created.
Installed at: ${SFSPATH}"
cp -a -f nvidia_${KERNELVER}-${NVVER}_amd64.sfs ${SFSPATH}/
cp -a -f nvidia_${KERNELVER}-${NVVER}_amd64.specs ${SFSPATH}/
cp -a -f nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/pixmaps/nvidia.png ${SFSPATH}/
sync

echo
echo "MUST SHUTDOWN WITHOUT SAVING SESSION!!!"
###end###
